import * as plugins from '../../../plugins.js';
import type { IAcmeOptions } from '../../../certificate/models/certificate-types.js';
import type { TForwardingType } from '../../../forwarding/config/forwarding-types.js';

/**
 * Supported action types for route configurations
 */
export type TRouteActionType = 'forward' | 'redirect' | 'block';

/**
 * TLS handling modes for route configurations
 */
export type TTlsMode = 'passthrough' | 'terminate' | 'terminate-and-reencrypt';

/**
 * Port range specification format
 */
export type TPortRange = number | number[] | Array<{ from: number; to: number }>;

/**
 * Route match criteria for incoming requests
 */
export interface IRouteMatch {
  // Listen on these ports (required)
  ports: TPortRange;
  
  // Optional domain patterns to match (default: all domains)
  domains?: string | string[];
  
  // Advanced matching criteria
  path?: string;           // Match specific paths
  clientIp?: string[];     // Match specific client IPs
  tlsVersion?: string[];   // Match specific TLS versions
}

/**
 * Target configuration for forwarding
 */
export interface IRouteTarget {
  host: string | string[];  // Support single host or round-robin
  port: number;
  preservePort?: boolean;   // Use incoming port as target port
}

/**
 * TLS configuration for route actions
 */
export interface IRouteTls {
  mode: TTlsMode;
  certificate?: 'auto' | {   // Auto = use ACME
    key: string;
    cert: string;
  };
}

/**
 * Redirect configuration for route actions
 */
export interface IRouteRedirect {
  to: string;            // URL or template with {domain}, {port}, etc.
  status: 301 | 302 | 307 | 308;
}

/**
 * Security options for route actions
 */
export interface IRouteSecurity {
  allowedIps?: string[];
  blockedIps?: string[];
  maxConnections?: number;
  authentication?: {
    type: 'basic' | 'digest' | 'oauth';
    // Auth-specific options would go here
  };
}

/**
 * Advanced options for route actions
 */
export interface IRouteAdvanced {
  timeout?: number;
  headers?: Record<string, string>;
  keepAlive?: boolean;
  // Additional advanced options would go here
}

/**
 * Action configuration for route handling
 */
export interface IRouteAction {
  // Basic routing
  type: TRouteActionType;
  
  // Target for forwarding
  target?: IRouteTarget;
  
  // TLS handling
  tls?: IRouteTls;
  
  // For redirects
  redirect?: IRouteRedirect;
  
  // Security options
  security?: IRouteSecurity;
  
  // Advanced options
  advanced?: IRouteAdvanced;
}

/**
 * The core unified configuration interface
 */
export interface IRouteConfig {
  // What to match
  match: IRouteMatch;
  
  // What to do with matched traffic
  action: IRouteAction;
  
  // Optional metadata
  name?: string;             // Human-readable name for this route
  description?: string;      // Description of the route's purpose
  priority?: number;         // Controls matching order (higher = matched first)
  tags?: string[];           // Arbitrary tags for categorization
}

/**
 * Unified SmartProxy options with routes-based configuration
 */
export interface IRoutedSmartProxyOptions {
  // The unified configuration array (required)
  routes: IRouteConfig[];
  
  // Global/default settings
  defaults?: {
    target?: {
      host: string;
      port: number;
    };
    security?: IRouteSecurity;
    tls?: IRouteTls;
    // ...other defaults
  };
  
  // Other global settings remain (acme, etc.)
  acme?: IAcmeOptions;
  
  // Connection timeouts and other global settings
  initialDataTimeout?: number;
  socketTimeout?: number;
  inactivityCheckInterval?: number;
  maxConnectionLifetime?: number;
  inactivityTimeout?: number;
  gracefulShutdownTimeout?: number;
  
  // Socket optimization settings
  noDelay?: boolean;
  keepAlive?: boolean;
  keepAliveInitialDelay?: number;
  maxPendingDataSize?: number;
  
  // Enhanced features
  disableInactivityCheck?: boolean;
  enableKeepAliveProbes?: boolean;
  enableDetailedLogging?: boolean;
  enableTlsDebugLogging?: boolean;
  enableRandomizedTimeouts?: boolean;
  allowSessionTicket?: boolean;
  
  // Rate limiting and security
  maxConnectionsPerIP?: number;
  connectionRateLimitPerMinute?: number;
  
  // Enhanced keep-alive settings
  keepAliveTreatment?: 'standard' | 'extended' | 'immortal';
  keepAliveInactivityMultiplier?: number;
  extendedKeepAliveLifetime?: number;
  
  /**
   * Optional certificate provider callback. Return 'http01' to use HTTP-01 challenges,
   * or a static certificate object for immediate provisioning.
   */
  certProvisionFunction?: (domain: string) => Promise<any>;
}